{% if access_lists %}

<table class="table table-hover">
    <tr>
        <th>Name</th>
        <th>Type</th>
        <th>Default Action</th>
        <th>Rule Count</th>
    </tr>
    {% for object in access_lists %}
        <tr>
            <td>{{ object|linkify }}</td>
            <td>{{ object.type|title }}</td>
            <td>{{ object.default_action|title }}</td>
            {% if object.type == 'standard' %}
            <td>{{ object.aclstandardrules.count|placeholder }}</td>
            {% elif object.type == 'extended' %}
            <td>{{ object.aclextendedrules.count|placeholder }}</td>
            {% endif %}
        </tr>
    {% endfor %}
</table>
{% else %}
<div class="text-muted">
    None found
</div>
{% endif %}
